# coding: utf-8

"""
    MoneyKit API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2023-02-18
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictStr
from pydantic import Field
from typing_extensions import Annotated
from moneykit.models.link_session_customer_user import LinkSessionCustomerUser
from moneykit.models.link_session_setting_overrides import LinkSessionSettingOverrides
from moneykit.models.money_kit_connect_features import MoneyKitConnectFeatures

try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class CreateLinkSessionRequest(BaseModel):
    """
    CreateLinkSessionRequest
    """  # noqa: E501

    settings: Optional[LinkSessionSettingOverrides] = None
    customer_user: LinkSessionCustomerUser
    existing_link_id: Optional[StrictStr] = Field(
        default=None,
        description="Supply the existing `link_id` if you are asking the user to reconnect this link.",
    )
    institution_id: Optional[StrictStr] = Field(
        default=None,
        description="The ID of the institution you want to link to. Providing this will skip the institution         selection step. `existing_link_id` will take precedence over this field if both are provided.",
    )
    redirect_uri: Annotated[
        str, Field(min_length=1, strict=True, max_length=65536)
    ] = Field(
        description="For Oauth linking, a URI indicating the destination, in your application, where the user should         be sent after authenticating with the institution.  The `redirect_uri` should not contain any query parameters,         and it must be pre-approved by MoneyKit during the customer setup process."
    )
    webhook: Optional[
        Annotated[str, Field(min_length=1, strict=True, max_length=65536)]
    ] = Field(
        default=None,
        description="The destination URL to which any webhooks should be sent.",
    )
    link_tags: Optional[List[StrictStr]] = None
    connect_features: Optional[MoneyKitConnectFeatures] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = [
        "settings",
        "customer_user",
        "existing_link_id",
        "institution_id",
        "redirect_uri",
        "webhook",
        "link_tags",
        "connect_features",
    ]

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of CreateLinkSessionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
                "additional_properties",
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of settings
        if self.settings:
            _dict["settings"] = self.settings.to_dict()
        # override the default output from pydantic by calling `to_dict()` of customer_user
        if self.customer_user:
            _dict["customer_user"] = self.customer_user.to_dict()
        # override the default output from pydantic by calling `to_dict()` of connect_features
        if self.connect_features:
            _dict["connect_features"] = self.connect_features.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of CreateLinkSessionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "settings": LinkSessionSettingOverrides.from_dict(obj.get("settings"))
                if obj.get("settings") is not None
                else None,
                "customer_user": LinkSessionCustomerUser.from_dict(
                    obj.get("customer_user")
                )
                if obj.get("customer_user") is not None
                else None,
                "existing_link_id": obj.get("existing_link_id"),
                "institution_id": obj.get("institution_id"),
                "redirect_uri": obj.get("redirect_uri"),
                "webhook": obj.get("webhook"),
                "link_tags": obj.get("link_tags"),
                "connect_features": MoneyKitConnectFeatures.from_dict(
                    obj.get("connect_features")
                )
                if obj.get("connect_features") is not None
                else None,
            }
        )
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



# coding: utf-8

"""
    MoneyKit API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2023-02-18
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501



from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Dict, List, Optional, Tuple, Union, Any

try:
    from typing import Annotated
except ImportError:
    from typing_extensions import Annotated

from pydantic import Field
from typing_extensions import Annotated
from datetime import date

from pydantic import StrictStr

from typing import List, Optional

from moneykit.models.get_transactions_response import GetTransactionsResponse
from moneykit.models.get_user_transactions_response import GetUserTransactionsResponse
from moneykit.models.transaction_sync_response import TransactionSyncResponse
from moneykit.models.transaction_type_filter import TransactionTypeFilter

from moneykit.api_client import ApiClient
from moneykit.api_response import ApiResponse
from moneykit.rest import RESTResponseType


class TransactionsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def get_transactions(
        self,
        id: Annotated[StrictStr, Field(description="The unique ID for this link.")],
        account_ids: Annotated[
            Optional[List[StrictStr]],
            Field(description="An optional list of account IDs to filter the results."),
        ] = None,
        page: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=1)]],
            Field(description="The page number to return."),
        ] = None,
        size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="The number of items to return per page."),
        ] = None,
        start_date: Annotated[
            Optional[date],
            Field(
                description="The earliest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to 90 days before the `end_date`.             <p>If you want to retrieve **all** transactions, use `1900-01-01`."
            ),
        ] = None,
        end_date: Annotated[
            Optional[date],
            Field(
                description="The latest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to today."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetTransactionsResponse:
        """/links/{id}/transactions

        Returns transactions for the accounts associated with a <a href=#tag/Links>link</a>.  Results are     paginated, and returned in reverse chronological order.     <p>**Note** that this endpoint does **not** trigger a fetch of transactions from the institution; it merely returns     transactions that have already been fetched, either because `prefetch` was requested when the link was created,     or because of scheduled or on-demand updates.     <p>MoneyKit checks for updated account data, including transactions, periodically throughout the day, but the     update frequency can vary, depending on the downstream data provider, the institution, and whether one or both     provide webhook-based updates.  **To force a check for updated transactions, you can use the     <a href=#operation/refresh_products>/products</a> endpoint.**     <p>If you have requested prefetch or an on-demand update, you should check the `refreshed_at` date     for this product in the returned response, and compare that against the previous `refreshed_at` date, which you can     get from any previous response for this or any other account or link request.  If the refreshed_at date has not     increased, then updated data is not yet available.

        :param id: The unique ID for this link. (required)
        :type id: str
        :param account_ids: An optional list of account IDs to filter the results.
        :type account_ids: List[str]
        :param page: The page number to return.
        :type page: int
        :param size: The number of items to return per page.
        :type size: int
        :param start_date: The earliest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to 90 days before the `end_date`.             <p>If you want to retrieve **all** transactions, use `1900-01-01`.
        :type start_date: date
        :param end_date: The latest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to today.
        :type end_date: date
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_transactions_serialize(
            id=id,
            account_ids=account_ids,
            page=page,
            size=size,
            start_date=start_date,
            end_date=end_date,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GetTransactionsResponse",
            "401": "Response401GetTransactions",
            "429": "APIErrorRateLimitExceededResponse",
            "404": "LinkErrorNotFoundResponse",
            "403": "LinkErrorForbiddenActionResponse",
            "410": "LinkErrorDeletedResponse",
            "422": "LinkErrorBadStateResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def get_transactions_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The unique ID for this link.")],
        account_ids: Annotated[
            Optional[List[StrictStr]],
            Field(description="An optional list of account IDs to filter the results."),
        ] = None,
        page: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=1)]],
            Field(description="The page number to return."),
        ] = None,
        size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="The number of items to return per page."),
        ] = None,
        start_date: Annotated[
            Optional[date],
            Field(
                description="The earliest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to 90 days before the `end_date`.             <p>If you want to retrieve **all** transactions, use `1900-01-01`."
            ),
        ] = None,
        end_date: Annotated[
            Optional[date],
            Field(
                description="The latest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to today."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetTransactionsResponse]:
        """/links/{id}/transactions

        Returns transactions for the accounts associated with a <a href=#tag/Links>link</a>.  Results are     paginated, and returned in reverse chronological order.     <p>**Note** that this endpoint does **not** trigger a fetch of transactions from the institution; it merely returns     transactions that have already been fetched, either because `prefetch` was requested when the link was created,     or because of scheduled or on-demand updates.     <p>MoneyKit checks for updated account data, including transactions, periodically throughout the day, but the     update frequency can vary, depending on the downstream data provider, the institution, and whether one or both     provide webhook-based updates.  **To force a check for updated transactions, you can use the     <a href=#operation/refresh_products>/products</a> endpoint.**     <p>If you have requested prefetch or an on-demand update, you should check the `refreshed_at` date     for this product in the returned response, and compare that against the previous `refreshed_at` date, which you can     get from any previous response for this or any other account or link request.  If the refreshed_at date has not     increased, then updated data is not yet available.

        :param id: The unique ID for this link. (required)
        :type id: str
        :param account_ids: An optional list of account IDs to filter the results.
        :type account_ids: List[str]
        :param page: The page number to return.
        :type page: int
        :param size: The number of items to return per page.
        :type size: int
        :param start_date: The earliest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to 90 days before the `end_date`.             <p>If you want to retrieve **all** transactions, use `1900-01-01`.
        :type start_date: date
        :param end_date: The latest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to today.
        :type end_date: date
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_transactions_serialize(
            id=id,
            account_ids=account_ids,
            page=page,
            size=size,
            start_date=start_date,
            end_date=end_date,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GetTransactionsResponse",
            "401": "Response401GetTransactions",
            "429": "APIErrorRateLimitExceededResponse",
            "404": "LinkErrorNotFoundResponse",
            "403": "LinkErrorForbiddenActionResponse",
            "410": "LinkErrorDeletedResponse",
            "422": "LinkErrorBadStateResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def get_transactions_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The unique ID for this link.")],
        account_ids: Annotated[
            Optional[List[StrictStr]],
            Field(description="An optional list of account IDs to filter the results."),
        ] = None,
        page: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=1)]],
            Field(description="The page number to return."),
        ] = None,
        size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="The number of items to return per page."),
        ] = None,
        start_date: Annotated[
            Optional[date],
            Field(
                description="The earliest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to 90 days before the `end_date`.             <p>If you want to retrieve **all** transactions, use `1900-01-01`."
            ),
        ] = None,
        end_date: Annotated[
            Optional[date],
            Field(
                description="The latest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to today."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """/links/{id}/transactions

        Returns transactions for the accounts associated with a <a href=#tag/Links>link</a>.  Results are     paginated, and returned in reverse chronological order.     <p>**Note** that this endpoint does **not** trigger a fetch of transactions from the institution; it merely returns     transactions that have already been fetched, either because `prefetch` was requested when the link was created,     or because of scheduled or on-demand updates.     <p>MoneyKit checks for updated account data, including transactions, periodically throughout the day, but the     update frequency can vary, depending on the downstream data provider, the institution, and whether one or both     provide webhook-based updates.  **To force a check for updated transactions, you can use the     <a href=#operation/refresh_products>/products</a> endpoint.**     <p>If you have requested prefetch or an on-demand update, you should check the `refreshed_at` date     for this product in the returned response, and compare that against the previous `refreshed_at` date, which you can     get from any previous response for this or any other account or link request.  If the refreshed_at date has not     increased, then updated data is not yet available.

        :param id: The unique ID for this link. (required)
        :type id: str
        :param account_ids: An optional list of account IDs to filter the results.
        :type account_ids: List[str]
        :param page: The page number to return.
        :type page: int
        :param size: The number of items to return per page.
        :type size: int
        :param start_date: The earliest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to 90 days before the `end_date`.             <p>If you want to retrieve **all** transactions, use `1900-01-01`.
        :type start_date: date
        :param end_date: The latest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to today.
        :type end_date: date
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_transactions_serialize(
            id=id,
            account_ids=account_ids,
            page=page,
            size=size,
            start_date=start_date,
            end_date=end_date,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GetTransactionsResponse",
            "401": "Response401GetTransactions",
            "429": "APIErrorRateLimitExceededResponse",
            "404": "LinkErrorNotFoundResponse",
            "403": "LinkErrorForbiddenActionResponse",
            "410": "LinkErrorDeletedResponse",
            "422": "LinkErrorBadStateResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_transactions_serialize(
        self,
        id,
        account_ids,
        page,
        size,
        start_date,
        end_date,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {
            "account_ids": "multi",
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params["id"] = id
        # process the query parameters
        if account_ids is not None:
            _query_params.append(("account_ids", account_ids))

        if page is not None:
            _query_params.append(("page", page))

        if size is not None:
            _query_params.append(("size", size))

        if start_date is not None:
            if isinstance(start_date, date):
                _query_params.append(
                    (
                        "start_date",
                        start_date.strftime(self.api_client.configuration.date_format),
                    )
                )
            else:
                _query_params.append(("start_date", start_date))

        if end_date is not None:
            if isinstance(end_date, date):
                _query_params.append(
                    (
                        "end_date",
                        end_date.strftime(self.api_client.configuration.date_format),
                    )
                )
            else:
                _query_params.append(("end_date", end_date))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # authentication setting
        _auth_settings: List[str] = ["OAuth2ClientCredentials"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/links/{id}/transactions",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def get_transactions_diff(
        self,
        id: Annotated[StrictStr, Field(description="The unique ID for this link.")],
        cursor: Annotated[
            Optional[StrictStr],
            Field(
                description="A cursor value representing the last update requested. If included, the response will only return         changes after this update. If omitted, a complete history of updates will be returned. This value must be stored         by the client as we do not keep track of customer cursors."
            ),
        ] = None,
        size: Annotated[
            Optional[Annotated[int, Field(le=500, strict=True, ge=1)]],
            Field(description="The number of items to return."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> TransactionSyncResponse:
        """/links/{id}/transactions/sync

        Provides a paginated feed of transactions, grouped into `created`, `updated`, and `removed` lists.         <p>Each call will also return a `cursor.next` value.  In subsequent calls, include that value to receive         only changes that have occurred since the previous call.         <p>Large numbers of transactions will be paginated, and the `has_more` field will be true.  You should         continue calling this endpoint with each new `cursor.next` value until `has_more` is false.         <p>**Note** that this endpoint does **not** trigger a fetch of transactions from the institution; it merely returns         transactions that have already been fetched, either because `prefetch` was requested when the link was created,         or because of scheduled or on-demand updates.         <p>MoneyKit checks for updated account data, including transactions, periodically throughout the day, but the         update frequency can vary, depending on the downstream data provider, the institution, and whether one or both         provide webhook-based updates.         **To force a check for updated transactions, you can use the <a href=#operation/refresh_products>/products</a> endpoint.**         <p>Note also that the `transactions.updates_available` webhook will alert you when new data is available.

        :param id: The unique ID for this link. (required)
        :type id: str
        :param cursor: A cursor value representing the last update requested. If included, the response will only return         changes after this update. If omitted, a complete history of updates will be returned. This value must be stored         by the client as we do not keep track of customer cursors.
        :type cursor: str
        :param size: The number of items to return.
        :type size: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_transactions_diff_serialize(
            id=id,
            cursor=cursor,
            size=size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "TransactionSyncResponse",
            "401": "Response401GetTransactionsDiffLinksIdTransactionsSyncGet",
            "429": "APIErrorRateLimitExceededResponse",
            "404": "LinkErrorNotFoundResponse",
            "403": "LinkErrorForbiddenActionResponse",
            "410": "LinkErrorDeletedResponse",
            "422": "LinkErrorBadStateResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def get_transactions_diff_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The unique ID for this link.")],
        cursor: Annotated[
            Optional[StrictStr],
            Field(
                description="A cursor value representing the last update requested. If included, the response will only return         changes after this update. If omitted, a complete history of updates will be returned. This value must be stored         by the client as we do not keep track of customer cursors."
            ),
        ] = None,
        size: Annotated[
            Optional[Annotated[int, Field(le=500, strict=True, ge=1)]],
            Field(description="The number of items to return."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[TransactionSyncResponse]:
        """/links/{id}/transactions/sync

        Provides a paginated feed of transactions, grouped into `created`, `updated`, and `removed` lists.         <p>Each call will also return a `cursor.next` value.  In subsequent calls, include that value to receive         only changes that have occurred since the previous call.         <p>Large numbers of transactions will be paginated, and the `has_more` field will be true.  You should         continue calling this endpoint with each new `cursor.next` value until `has_more` is false.         <p>**Note** that this endpoint does **not** trigger a fetch of transactions from the institution; it merely returns         transactions that have already been fetched, either because `prefetch` was requested when the link was created,         or because of scheduled or on-demand updates.         <p>MoneyKit checks for updated account data, including transactions, periodically throughout the day, but the         update frequency can vary, depending on the downstream data provider, the institution, and whether one or both         provide webhook-based updates.         **To force a check for updated transactions, you can use the <a href=#operation/refresh_products>/products</a> endpoint.**         <p>Note also that the `transactions.updates_available` webhook will alert you when new data is available.

        :param id: The unique ID for this link. (required)
        :type id: str
        :param cursor: A cursor value representing the last update requested. If included, the response will only return         changes after this update. If omitted, a complete history of updates will be returned. This value must be stored         by the client as we do not keep track of customer cursors.
        :type cursor: str
        :param size: The number of items to return.
        :type size: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_transactions_diff_serialize(
            id=id,
            cursor=cursor,
            size=size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "TransactionSyncResponse",
            "401": "Response401GetTransactionsDiffLinksIdTransactionsSyncGet",
            "429": "APIErrorRateLimitExceededResponse",
            "404": "LinkErrorNotFoundResponse",
            "403": "LinkErrorForbiddenActionResponse",
            "410": "LinkErrorDeletedResponse",
            "422": "LinkErrorBadStateResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def get_transactions_diff_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The unique ID for this link.")],
        cursor: Annotated[
            Optional[StrictStr],
            Field(
                description="A cursor value representing the last update requested. If included, the response will only return         changes after this update. If omitted, a complete history of updates will be returned. This value must be stored         by the client as we do not keep track of customer cursors."
            ),
        ] = None,
        size: Annotated[
            Optional[Annotated[int, Field(le=500, strict=True, ge=1)]],
            Field(description="The number of items to return."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """/links/{id}/transactions/sync

        Provides a paginated feed of transactions, grouped into `created`, `updated`, and `removed` lists.         <p>Each call will also return a `cursor.next` value.  In subsequent calls, include that value to receive         only changes that have occurred since the previous call.         <p>Large numbers of transactions will be paginated, and the `has_more` field will be true.  You should         continue calling this endpoint with each new `cursor.next` value until `has_more` is false.         <p>**Note** that this endpoint does **not** trigger a fetch of transactions from the institution; it merely returns         transactions that have already been fetched, either because `prefetch` was requested when the link was created,         or because of scheduled or on-demand updates.         <p>MoneyKit checks for updated account data, including transactions, periodically throughout the day, but the         update frequency can vary, depending on the downstream data provider, the institution, and whether one or both         provide webhook-based updates.         **To force a check for updated transactions, you can use the <a href=#operation/refresh_products>/products</a> endpoint.**         <p>Note also that the `transactions.updates_available` webhook will alert you when new data is available.

        :param id: The unique ID for this link. (required)
        :type id: str
        :param cursor: A cursor value representing the last update requested. If included, the response will only return         changes after this update. If omitted, a complete history of updates will be returned. This value must be stored         by the client as we do not keep track of customer cursors.
        :type cursor: str
        :param size: The number of items to return.
        :type size: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_transactions_diff_serialize(
            id=id,
            cursor=cursor,
            size=size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "TransactionSyncResponse",
            "401": "Response401GetTransactionsDiffLinksIdTransactionsSyncGet",
            "429": "APIErrorRateLimitExceededResponse",
            "404": "LinkErrorNotFoundResponse",
            "403": "LinkErrorForbiddenActionResponse",
            "410": "LinkErrorDeletedResponse",
            "422": "LinkErrorBadStateResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_transactions_diff_serialize(
        self,
        id,
        cursor,
        size,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params["id"] = id
        # process the query parameters
        if cursor is not None:
            _query_params.append(("cursor", cursor))

        if size is not None:
            _query_params.append(("size", size))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # authentication setting
        _auth_settings: List[str] = ["OAuth2ClientCredentials"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/links/{id}/transactions/sync",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def get_user_transactions(
        self,
        id: Annotated[
            StrictStr,
            Field(
                description="The unique ID for this user.  This is the same ID provided         in the call to <a href=#operation/create_link_session>/link-session</a> to create any link for this user."
            ),
        ],
        transaction_type: Optional[List[TransactionTypeFilter]] = None,
        category: Optional[List[StrictStr]] = None,
        account_id: Annotated[
            Optional[List[Annotated[str, Field(min_length=1, strict=True)]]],
            Field(
                description="If present, filters results to transactions in accounts matching the given IDs."
            ),
        ] = None,
        institution_id: Annotated[
            Optional[List[Annotated[str, Field(min_length=1, strict=True)]]],
            Field(
                description="If present, filters results to transactions at institutions matching the given IDs."
            ),
        ] = None,
        page: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=1)]],
            Field(description="The page number to return."),
        ] = None,
        size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="The number of items to return per page."),
        ] = None,
        start_date: Annotated[
            Optional[date],
            Field(
                description="The earliest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to 90 days before the `end_date`.             <p>If you want to retrieve **all** transactions, use `1900-01-01`."
            ),
        ] = None,
        end_date: Annotated[
            Optional[date],
            Field(
                description="The latest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to today."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetUserTransactionsResponse:
        """/users/{id}/transactions

        Fetches transactions for a <a href=#operation/get_user_accounts>user</a>.     <p>This endpoint fetches all transactions for a user across all of their links.  You can use it to retrieve     transactions from any or all accounts at once, regardless of which institution they belong to.

        :param id: The unique ID for this user.  This is the same ID provided         in the call to <a href=#operation/create_link_session>/link-session</a> to create any link for this user. (required)
        :type id: str
        :param transaction_type:
        :type transaction_type: List[TransactionTypeFilter]
        :param category:
        :type category: List[str]
        :param account_id: If present, filters results to transactions in accounts matching the given IDs.
        :type account_id: List[str]
        :param institution_id: If present, filters results to transactions at institutions matching the given IDs.
        :type institution_id: List[str]
        :param page: The page number to return.
        :type page: int
        :param size: The number of items to return per page.
        :type size: int
        :param start_date: The earliest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to 90 days before the `end_date`.             <p>If you want to retrieve **all** transactions, use `1900-01-01`.
        :type start_date: date
        :param end_date: The latest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to today.
        :type end_date: date
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_user_transactions_serialize(
            id=id,
            transaction_type=transaction_type,
            category=category,
            account_id=account_id,
            institution_id=institution_id,
            page=page,
            size=size,
            start_date=start_date,
            end_date=end_date,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GetUserTransactionsResponse",
            "401": "Response401GetUserTransactions",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def get_user_transactions_with_http_info(
        self,
        id: Annotated[
            StrictStr,
            Field(
                description="The unique ID for this user.  This is the same ID provided         in the call to <a href=#operation/create_link_session>/link-session</a> to create any link for this user."
            ),
        ],
        transaction_type: Optional[List[TransactionTypeFilter]] = None,
        category: Optional[List[StrictStr]] = None,
        account_id: Annotated[
            Optional[List[Annotated[str, Field(min_length=1, strict=True)]]],
            Field(
                description="If present, filters results to transactions in accounts matching the given IDs."
            ),
        ] = None,
        institution_id: Annotated[
            Optional[List[Annotated[str, Field(min_length=1, strict=True)]]],
            Field(
                description="If present, filters results to transactions at institutions matching the given IDs."
            ),
        ] = None,
        page: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=1)]],
            Field(description="The page number to return."),
        ] = None,
        size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="The number of items to return per page."),
        ] = None,
        start_date: Annotated[
            Optional[date],
            Field(
                description="The earliest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to 90 days before the `end_date`.             <p>If you want to retrieve **all** transactions, use `1900-01-01`."
            ),
        ] = None,
        end_date: Annotated[
            Optional[date],
            Field(
                description="The latest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to today."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetUserTransactionsResponse]:
        """/users/{id}/transactions

        Fetches transactions for a <a href=#operation/get_user_accounts>user</a>.     <p>This endpoint fetches all transactions for a user across all of their links.  You can use it to retrieve     transactions from any or all accounts at once, regardless of which institution they belong to.

        :param id: The unique ID for this user.  This is the same ID provided         in the call to <a href=#operation/create_link_session>/link-session</a> to create any link for this user. (required)
        :type id: str
        :param transaction_type:
        :type transaction_type: List[TransactionTypeFilter]
        :param category:
        :type category: List[str]
        :param account_id: If present, filters results to transactions in accounts matching the given IDs.
        :type account_id: List[str]
        :param institution_id: If present, filters results to transactions at institutions matching the given IDs.
        :type institution_id: List[str]
        :param page: The page number to return.
        :type page: int
        :param size: The number of items to return per page.
        :type size: int
        :param start_date: The earliest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to 90 days before the `end_date`.             <p>If you want to retrieve **all** transactions, use `1900-01-01`.
        :type start_date: date
        :param end_date: The latest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to today.
        :type end_date: date
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_user_transactions_serialize(
            id=id,
            transaction_type=transaction_type,
            category=category,
            account_id=account_id,
            institution_id=institution_id,
            page=page,
            size=size,
            start_date=start_date,
            end_date=end_date,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GetUserTransactionsResponse",
            "401": "Response401GetUserTransactions",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def get_user_transactions_without_preload_content(
        self,
        id: Annotated[
            StrictStr,
            Field(
                description="The unique ID for this user.  This is the same ID provided         in the call to <a href=#operation/create_link_session>/link-session</a> to create any link for this user."
            ),
        ],
        transaction_type: Optional[List[TransactionTypeFilter]] = None,
        category: Optional[List[StrictStr]] = None,
        account_id: Annotated[
            Optional[List[Annotated[str, Field(min_length=1, strict=True)]]],
            Field(
                description="If present, filters results to transactions in accounts matching the given IDs."
            ),
        ] = None,
        institution_id: Annotated[
            Optional[List[Annotated[str, Field(min_length=1, strict=True)]]],
            Field(
                description="If present, filters results to transactions at institutions matching the given IDs."
            ),
        ] = None,
        page: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=1)]],
            Field(description="The page number to return."),
        ] = None,
        size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="The number of items to return per page."),
        ] = None,
        start_date: Annotated[
            Optional[date],
            Field(
                description="The earliest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to 90 days before the `end_date`.             <p>If you want to retrieve **all** transactions, use `1900-01-01`."
            ),
        ] = None,
        end_date: Annotated[
            Optional[date],
            Field(
                description="The latest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to today."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """/users/{id}/transactions

        Fetches transactions for a <a href=#operation/get_user_accounts>user</a>.     <p>This endpoint fetches all transactions for a user across all of their links.  You can use it to retrieve     transactions from any or all accounts at once, regardless of which institution they belong to.

        :param id: The unique ID for this user.  This is the same ID provided         in the call to <a href=#operation/create_link_session>/link-session</a> to create any link for this user. (required)
        :type id: str
        :param transaction_type:
        :type transaction_type: List[TransactionTypeFilter]
        :param category:
        :type category: List[str]
        :param account_id: If present, filters results to transactions in accounts matching the given IDs.
        :type account_id: List[str]
        :param institution_id: If present, filters results to transactions at institutions matching the given IDs.
        :type institution_id: List[str]
        :param page: The page number to return.
        :type page: int
        :param size: The number of items to return per page.
        :type size: int
        :param start_date: The earliest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to 90 days before the `end_date`.             <p>If you want to retrieve **all** transactions, use `1900-01-01`.
        :type start_date: date
        :param end_date: The latest date for which data should be returned, formatted as YYYY-MM-DD.             Defaults to today.
        :type end_date: date
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_user_transactions_serialize(
            id=id,
            transaction_type=transaction_type,
            category=category,
            account_id=account_id,
            institution_id=institution_id,
            page=page,
            size=size,
            start_date=start_date,
            end_date=end_date,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GetUserTransactionsResponse",
            "401": "Response401GetUserTransactions",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_user_transactions_serialize(
        self,
        id,
        transaction_type,
        category,
        account_id,
        institution_id,
        page,
        size,
        start_date,
        end_date,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {
            "transaction_type": "multi",
            "category": "multi",
            "account_id": "multi",
            "institution_id": "multi",
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params["id"] = id
        # process the query parameters
        if transaction_type is not None:
            _query_params.append(("transaction_type", transaction_type))

        if category is not None:
            _query_params.append(("category", category))

        if account_id is not None:
            _query_params.append(("account_id", account_id))

        if institution_id is not None:
            _query_params.append(("institution_id", institution_id))

        if page is not None:
            _query_params.append(("page", page))

        if size is not None:
            _query_params.append(("size", size))

        if start_date is not None:
            if isinstance(start_date, date):
                _query_params.append(
                    (
                        "start_date",
                        start_date.strftime(self.api_client.configuration.date_format),
                    )
                )
            else:
                _query_params.append(("start_date", start_date))

        if end_date is not None:
            if isinstance(end_date, date):
                _query_params.append(
                    (
                        "end_date",
                        end_date.strftime(self.api_client.configuration.date_format),
                    )
                )
            else:
                _query_params.append(("end_date", end_date))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # authentication setting
        _auth_settings: List[str] = ["OAuth2ClientCredentials"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/users/{id}/transactions",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



# coding: utf-8

"""
    MoneyKit API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2023-02-18
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictStr, field_validator

try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class LinkErrorUnauthorizedAccessResponse(BaseModel):
    """
    LinkErrorUnauthorizedAccessResponse
    """  # noqa: E501

    error_code: Optional[StrictStr] = "link_error.unauthorized_access"
    error_message: Optional[StrictStr] = "Unauthorized link access."
    documentation_url: Optional[StrictStr] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = [
        "error_code",
        "error_message",
        "documentation_url",
    ]

    @field_validator("error_code")
    def error_code_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ("link_error.unauthorized_access",):
            raise ValueError(
                "must be one of enum values ('link_error.unauthorized_access')"
            )
        return value

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of LinkErrorUnauthorizedAccessResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
                "additional_properties",
            },
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of LinkErrorUnauthorizedAccessResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "error_code": obj.get("error_code")
                if obj.get("error_code") is not None
                else "link_error.unauthorized_access",
                "error_message": obj.get("error_message")
                if obj.get("error_message") is not None
                else "Unauthorized link access.",
                "documentation_url": obj.get("documentation_url"),
            }
        )
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



# coding: utf-8

# flake8: noqa

"""
    MoneyKit API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2023-02-18
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "0.1.7"

# import apis into sdk package
from moneykit.api.access_token_api import AccessTokenApi
from moneykit.api.account_numbers_api import AccountNumbersApi
from moneykit.api.accounts_api import AccountsApi
from moneykit.api.identity_api import IdentityApi
from moneykit.api.institutions_api import InstitutionsApi
from moneykit.api.investments_api import InvestmentsApi
from moneykit.api.link_session_api import LinkSessionApi
from moneykit.api.links_api import LinksApi
from moneykit.api.products_api import ProductsApi
from moneykit.api.transactions_api import TransactionsApi
from moneykit.api.users_api import UsersApi
from moneykit.api.webhooks_api import WebhooksApi

# import ApiClient
from moneykit.api_response import ApiResponse
from moneykit.api_client import ApiClient
from moneykit.configuration import Configuration
from moneykit.exceptions import OpenApiException
from moneykit.exceptions import ApiTypeError
from moneykit.exceptions import ApiValueError
from moneykit.exceptions import ApiKeyError
from moneykit.exceptions import ApiAttributeError
from moneykit.exceptions import ApiException

# import models into sdk package
from moneykit.models.api_error_auth_expired_access_token_response import (
    APIErrorAuthExpiredAccessTokenResponse,
)
from moneykit.models.api_error_auth_unauthorized_response import (
    APIErrorAuthUnauthorizedResponse,
)
from moneykit.models.api_error_rate_limit_exceeded_response import (
    APIErrorRateLimitExceededResponse,
)
from moneykit.models.account import Account
from moneykit.models.account_balances import AccountBalances
from moneykit.models.account_group import AccountGroup
from moneykit.models.account_identity import AccountIdentity
from moneykit.models.account_numbers import AccountNumbers
from moneykit.models.account_numbers_link_product import AccountNumbersLinkProduct
from moneykit.models.account_numbers_product_settings import (
    AccountNumbersProductSettings,
)
from moneykit.models.account_with_account_numbers import AccountWithAccountNumbers
from moneykit.models.accounts_link_product import AccountsLinkProduct
from moneykit.models.ach_number import AchNumber
from moneykit.models.address import Address
from moneykit.models.bacs_number import BacsNumber
from moneykit.models.basic_account_details import BasicAccountDetails
from moneykit.models.body import Body
from moneykit.models.country import Country
from moneykit.models.create_link_session_request import CreateLinkSessionRequest
from moneykit.models.create_link_session_response import CreateLinkSessionResponse
from moneykit.models.currency import Currency
from moneykit.models.cursor_pagination import CursorPagination
from moneykit.models.customer_app import CustomerApp
from moneykit.models.eft_number import EftNumber
from moneykit.models.email import Email
from moneykit.models.exchange_token_request import ExchangeTokenRequest
from moneykit.models.exchange_token_response import ExchangeTokenResponse
from moneykit.models.generate_access_token_response import GenerateAccessTokenResponse
from moneykit.models.get_account_numbers_response import GetAccountNumbersResponse
from moneykit.models.get_account_response import GetAccountResponse
from moneykit.models.get_accounts_response import GetAccountsResponse
from moneykit.models.get_holdings_response import GetHoldingsResponse
from moneykit.models.get_institutions_response import GetInstitutionsResponse
from moneykit.models.get_investment_transactions_response import (
    GetInvestmentTransactionsResponse,
)
from moneykit.models.get_transactions_response import GetTransactionsResponse
from moneykit.models.get_user_accounts_response import GetUserAccountsResponse
from moneykit.models.get_user_links_response import GetUserLinksResponse
from moneykit.models.get_user_transactions_response import GetUserTransactionsResponse
from moneykit.models.http_validation_error import HTTPValidationError
from moneykit.models.holding_response import HoldingResponse
from moneykit.models.identity_link_product import IdentityLinkProduct
from moneykit.models.identity_product_settings import IdentityProductSettings
from moneykit.models.identity_response import IdentityResponse
from moneykit.models.institution import Institution
from moneykit.models.institution_error_not_found_response import (
    InstitutionErrorNotFoundResponse,
)
from moneykit.models.international_number import InternationalNumber
from moneykit.models.introspect_client_response import IntrospectClientResponse
from moneykit.models.investment_transaction_response import (
    InvestmentTransactionResponse,
)
from moneykit.models.investments_link_product import InvestmentsLinkProduct
from moneykit.models.investments_product_settings import InvestmentsProductSettings
from moneykit.models.jwk_set import JWKSet
from moneykit.models.link_common import LinkCommon
from moneykit.models.link_error import LinkError
from moneykit.models.link_error_bad_config_response import LinkErrorBadConfigResponse
from moneykit.models.link_error_bad_state_response import LinkErrorBadStateResponse
from moneykit.models.link_error_deleted_response import LinkErrorDeletedResponse
from moneykit.models.link_error_forbidden_action_response import (
    LinkErrorForbiddenActionResponse,
)
from moneykit.models.link_error_not_found_response import LinkErrorNotFoundResponse
from moneykit.models.link_error_unauthorized_access_response import (
    LinkErrorUnauthorizedAccessResponse,
)
from moneykit.models.link_permission_scope import LinkPermissionScope
from moneykit.models.link_permissions import LinkPermissions
from moneykit.models.link_product_failure_reasons import LinkProductFailureReasons
from moneykit.models.link_product_refresh_webhook import LinkProductRefreshWebhook
from moneykit.models.link_product_state import LinkProductState
from moneykit.models.link_products import LinkProducts
from moneykit.models.link_response import LinkResponse
from moneykit.models.link_session_customer_user import LinkSessionCustomerUser
from moneykit.models.link_session_customer_user_email import (
    LinkSessionCustomerUserEmail,
)
from moneykit.models.link_session_customer_user_phone import (
    LinkSessionCustomerUserPhone,
)
from moneykit.models.link_session_error_forbidden_config_response import (
    LinkSessionErrorForbiddenConfigResponse,
)
from moneykit.models.link_session_error_invalid_token_exchange import (
    LinkSessionErrorInvalidTokenExchange,
)
from moneykit.models.link_session_setting_overrides import LinkSessionSettingOverrides
from moneykit.models.link_state import LinkState
from moneykit.models.link_state_changed_webhook import LinkStateChangedWebhook
from moneykit.models.money_kit_connect_features import MoneyKitConnectFeatures
from moneykit.models.owner import Owner
from moneykit.models.phone_number import PhoneNumber
from moneykit.models.phone_number_type import PhoneNumberType
from moneykit.models.product import Product
from moneykit.models.products_settings import ProductsSettings
from moneykit.models.provider import Provider
from moneykit.models.refresh_products_request import RefreshProductsRequest
from moneykit.models.requested_link_permission import RequestedLinkPermission
from moneykit.models.response401_disconnect_links_id_delete import (
    Response401DisconnectLinksIdDelete,
)
from moneykit.models.response401_exchange_token_link_session_exchange_token_post import (
    Response401ExchangeTokenLinkSessionExchangeTokenPost,
)
from moneykit.models.response401_get_account_links_id_accounts_account_id_get import (
    Response401GetAccountLinksIdAccountsAccountIdGet,
)
from moneykit.models.response401_get_account_numbers_links_id_accounts_numbers_get import (
    Response401GetAccountNumbersLinksIdAccountsNumbersGet,
)
from moneykit.models.response401_get_accounts_links_id_accounts_get import (
    Response401GetAccountsLinksIdAccountsGet,
)
from moneykit.models.response401_get_holdings_links_id_investments_holdings_get import (
    Response401GetHoldingsLinksIdInvestmentsHoldingsGet,
)
from moneykit.models.response401_get_identities_links_id_identity_get import (
    Response401GetIdentitiesLinksIdIdentityGet,
)
from moneykit.models.response401_get_institution_institutions_institution_id_get import (
    Response401GetInstitutionInstitutionsInstitutionIdGet,
)
from moneykit.models.response401_get_institutions_institutions_get import (
    Response401GetInstitutionsInstitutionsGet,
)
from moneykit.models.response401_get_investment_transactions_links_id_investments_transactions_get import (
    Response401GetInvestmentTransactionsLinksIdInvestmentsTransactionsGet,
)
from moneykit.models.response401_get_link_links_id_get import (
    Response401GetLinkLinksIdGet,
)
from moneykit.models.response401_get_transactions import Response401GetTransactions
from moneykit.models.response401_get_transactions_diff_links_id_transactions_sync_get import (
    Response401GetTransactionsDiffLinksIdTransactionsSyncGet,
)
from moneykit.models.response401_get_user_accounts_users_id_accounts_get import (
    Response401GetUserAccountsUsersIdAccountsGet,
)
from moneykit.models.response401_get_user_links_users_id_links_get import (
    Response401GetUserLinksUsersIdLinksGet,
)
from moneykit.models.response401_get_user_transactions import (
    Response401GetUserTransactions,
)
from moneykit.models.response401_get_well_known_jwks_well_known_jwks_json_get import (
    Response401GetWellKnownJwksWellKnownJwksJsonGet,
)
from moneykit.models.response401_instrospect_client_auth_introspect_get import (
    Response401InstrospectClientAuthIntrospectGet,
)
from moneykit.models.response401_refresh_products_links_id_products_post import (
    Response401RefreshProductsLinksIdProductsPost,
)
from moneykit.models.response401_reset_login_links_id_reset_post import (
    Response401ResetLoginLinksIdResetPost,
)
from moneykit.models.response401_trigger_test_link_webhook_event_webhooks_test_link_id_post import (
    Response401TriggerTestLinkWebhookEventWebhooksTestLinkIdPost,
)
from moneykit.models.response401_update_link_links_id_patch import (
    Response401UpdateLinkLinksIdPatch,
)
from moneykit.models.response_handle_link_webhook_event_request_body_webhook_post import (
    ResponseHandleLinkWebhookEventRequestBodyWebhookPost,
)
from moneykit.models.security_response import SecurityResponse
from moneykit.models.transaction import Transaction
from moneykit.models.transaction_diff import TransactionDiff
from moneykit.models.transaction_sync_response import TransactionSyncResponse
from moneykit.models.transaction_type import TransactionType
from moneykit.models.transaction_type_filter import TransactionTypeFilter
from moneykit.models.transaction_updates_available_webhook import (
    TransactionUpdatesAvailableWebhook,
)
from moneykit.models.transactions_link_product import TransactionsLinkProduct
from moneykit.models.transactions_product_settings import TransactionsProductSettings
from moneykit.models.update_link_request import UpdateLinkRequest
from moneykit.models.validation_error import ValidationError
from moneykit.models.validation_error_location_inner import ValidationErrorLocationInner
from moneykit.models.webhook_link_test_event import WebhookLinkTestEvent
from moneykit.models.webhook_test_link_request import WebhookTestLinkRequest
from moneykit.models.webhook_test_link_response import WebhookTestLinkResponse



# coding: utf-8

"""
    MoneyKit API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2023-02-18
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501



from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Dict, List, Optional, Tuple, Union, Any

try:
    from typing import Annotated
except ImportError:
    from typing_extensions import Annotated

from moneykit.models.create_link_session_request import CreateLinkSessionRequest
from moneykit.models.create_link_session_response import CreateLinkSessionResponse
from moneykit.models.exchange_token_request import ExchangeTokenRequest
from moneykit.models.exchange_token_response import ExchangeTokenResponse

from moneykit.api_client import ApiClient
from moneykit.api_response import ApiResponse
from moneykit.rest import RESTResponseType


class LinkSessionApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def create_link_session(
        self,
        create_link_session_request: CreateLinkSessionRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CreateLinkSessionResponse:
        """/link-session

        This endpoint is to be called by your back end, to establish a new link session         for creating a link to your end user's institution.

        :param create_link_session_request: (required)
        :type create_link_session_request: CreateLinkSessionRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_link_session_serialize(
            create_link_session_request=create_link_session_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "CreateLinkSessionResponse",
            "401": "AnyOfAPIErrorAuthExpiredAccessTokenResponseAPIErrorAuthUnauthorizedResponse",
            "403": "LinkSessionErrorForbiddenConfigResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def create_link_session_with_http_info(
        self,
        create_link_session_request: CreateLinkSessionRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CreateLinkSessionResponse]:
        """/link-session

        This endpoint is to be called by your back end, to establish a new link session         for creating a link to your end user's institution.

        :param create_link_session_request: (required)
        :type create_link_session_request: CreateLinkSessionRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_link_session_serialize(
            create_link_session_request=create_link_session_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "CreateLinkSessionResponse",
            "401": "AnyOfAPIErrorAuthExpiredAccessTokenResponseAPIErrorAuthUnauthorizedResponse",
            "403": "LinkSessionErrorForbiddenConfigResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def create_link_session_without_preload_content(
        self,
        create_link_session_request: CreateLinkSessionRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """/link-session

        This endpoint is to be called by your back end, to establish a new link session         for creating a link to your end user's institution.

        :param create_link_session_request: (required)
        :type create_link_session_request: CreateLinkSessionRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_link_session_serialize(
            create_link_session_request=create_link_session_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "CreateLinkSessionResponse",
            "401": "AnyOfAPIErrorAuthExpiredAccessTokenResponseAPIErrorAuthUnauthorizedResponse",
            "403": "LinkSessionErrorForbiddenConfigResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _create_link_session_serialize(
        self,
        create_link_session_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if create_link_session_request is not None:
            _body_params = create_link_session_request

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ["application/json"]
            )
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["OAuth2ClientCredentials"]

        return self.api_client.param_serialize(
            method="POST",
            resource_path="/link-session",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def exchange_token(
        self,
        exchange_token_request: ExchangeTokenRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ExchangeTokenResponse:
        """/link-session/exchange-token

        After the end user has successfully completed the linking process, your back end     calls this endpoint to exchange the token received by your front end for a`link_id` that can be used to access     the link's data.

        :param exchange_token_request: (required)
        :type exchange_token_request: ExchangeTokenRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._exchange_token_serialize(
            exchange_token_request=exchange_token_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "ExchangeTokenResponse",
            "401": "Response401ExchangeTokenLinkSessionExchangeTokenPost",
            "400": "LinkSessionErrorInvalidTokenExchange",
            "422": "LinkSessionErrorInvalidTokenExchange",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def exchange_token_with_http_info(
        self,
        exchange_token_request: ExchangeTokenRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ExchangeTokenResponse]:
        """/link-session/exchange-token

        After the end user has successfully completed the linking process, your back end     calls this endpoint to exchange the token received by your front end for a`link_id` that can be used to access     the link's data.

        :param exchange_token_request: (required)
        :type exchange_token_request: ExchangeTokenRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._exchange_token_serialize(
            exchange_token_request=exchange_token_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "ExchangeTokenResponse",
            "401": "Response401ExchangeTokenLinkSessionExchangeTokenPost",
            "400": "LinkSessionErrorInvalidTokenExchange",
            "422": "LinkSessionErrorInvalidTokenExchange",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def exchange_token_without_preload_content(
        self,
        exchange_token_request: ExchangeTokenRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """/link-session/exchange-token

        After the end user has successfully completed the linking process, your back end     calls this endpoint to exchange the token received by your front end for a`link_id` that can be used to access     the link's data.

        :param exchange_token_request: (required)
        :type exchange_token_request: ExchangeTokenRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._exchange_token_serialize(
            exchange_token_request=exchange_token_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "ExchangeTokenResponse",
            "401": "Response401ExchangeTokenLinkSessionExchangeTokenPost",
            "400": "LinkSessionErrorInvalidTokenExchange",
            "422": "LinkSessionErrorInvalidTokenExchange",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _exchange_token_serialize(
        self,
        exchange_token_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if exchange_token_request is not None:
            _body_params = exchange_token_request

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ["application/json"]
            )
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["OAuth2ClientCredentials"]

        return self.api_client.param_serialize(
            method="POST",
            resource_path="/link-session/exchange-token",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



# coding: utf-8

"""
    MoneyKit API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2023-02-18
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictStr
from pydantic import Field

try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class SecurityResponse(BaseModel):
    """
    SecurityResponse
    """  # noqa: E501

    id: StrictStr = Field(description="A unique MoneyKit ID for this security.")
    name: StrictStr = Field(description="A user-facing name for this security.")
    type: StrictStr = Field(
        description="The type of this security.  Will be one of: <p>`cash`: Cash, currency, and money market funds <p>`crypto`: Digital or virtual currencies <p>`derivative`: Options, warrants, and other derivative instruments <p>`equity`: Equities (stocks) <p>`etf`: Exchange-traded investment funds <p>`bond`: Bonds and certificates of deposit (CDs) <p>`loan`: Loans and loan receivables <p>`fund`: Open- and closed-end vehicles pooling funds of multiple investors <p>`unknown`: Indicates the type is unknown or does not fit into any other category"
    )
    currency: StrictStr = Field(
        description="The ISO-4217 code for the currency in which this security is priced."
    )
    ticker: Optional[StrictStr] = Field(
        default=None, description="The security's public trading symbol, if available."
    )
    isin: Optional[StrictStr] = Field(
        default=None,
        description="The ISO 6166 globally unique ID for this security, if available.",
    )
    sedol: Optional[StrictStr] = Field(
        default=None,
        description="The security's SEDOL ID from the London Stock Exchange, if available.",
    )
    close_price: Optional[StrictStr] = Field(
        default=None,
        description="The price of the security at the close of the previous trading session, given in pricing currency.",
    )
    close_date: Optional[date] = Field(
        default=None, description="The date for which `close_price` is given."
    )
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = [
        "id",
        "name",
        "type",
        "currency",
        "ticker",
        "isin",
        "sedol",
        "close_price",
        "close_date",
    ]

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SecurityResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
                "additional_properties",
            },
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of SecurityResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "name": obj.get("name"),
                "type": obj.get("type"),
                "currency": obj.get("currency"),
                "ticker": obj.get("ticker"),
                "isin": obj.get("isin"),
                "sedol": obj.get("sedol"),
                "close_price": obj.get("close_price"),
                "close_date": obj.get("close_date"),
            }
        )
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



# flake8: noqa

# import apis into api package
from moneykit.api.access_token_api import AccessTokenApi
from moneykit.api.account_numbers_api import AccountNumbersApi
from moneykit.api.accounts_api import AccountsApi
from moneykit.api.identity_api import IdentityApi
from moneykit.api.institutions_api import InstitutionsApi
from moneykit.api.investments_api import InvestmentsApi
from moneykit.api.link_session_api import LinkSessionApi
from moneykit.api.links_api import LinksApi
from moneykit.api.products_api import ProductsApi
from moneykit.api.transactions_api import TransactionsApi
from moneykit.api.users_api import UsersApi
from moneykit.api.webhooks_api import WebhooksApi



# coding: utf-8

"""
    MoneyKit API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2023-02-18
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from datetime import datetime
from typing import Any, ClassVar, Dict, List, Literal, Optional

from pydantic import BaseModel, StrictInt, StrictStr, field_validator

from moneykit.models.link_error import LinkError
from moneykit.models.link_state import LinkState

try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class LinkStateChangedWebhook(BaseModel):
    """
    LinkStateChangedWebhook
    """  # noqa: E501

    webhook_event: Literal["link.state_changed"] = "link.state_changed"
    webhook_major_version: int = 1
    webhook_minor_version: int = 0
    webhook_idempotency_key: StrictStr
    webhook_timestamp: datetime
    link_id: StrictStr
    link_tags: List[StrictStr]
    state: LinkState
    error: Optional[LinkError] = None
    error_message: Optional[StrictStr] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = [
        "webhook_event",
        "webhook_major_version",
        "webhook_minor_version",
        "webhook_idempotency_key",
        "webhook_timestamp",
        "link_id",
        "link_tags",
        "state",
        "error",
        "error_message",
    ]

    @field_validator("webhook_event")
    def webhook_event_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ("link.state_changed",):
            raise ValueError("must be one of enum values ('link.state_changed')")
        return value

    @field_validator("webhook_major_version")
    def webhook_major_version_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (1,):
            raise ValueError("must be one of enum values (1)")
        return value

    @field_validator("webhook_minor_version")
    def webhook_minor_version_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (0,):
            raise ValueError("must be one of enum values (0)")
        return value

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of LinkStateChangedWebhook from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
                "additional_properties",
            },
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of LinkStateChangedWebhook from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "webhook_event": obj.get("webhook_event")
                if obj.get("webhook_event") is not None
                else "link.state_changed",
                "webhook_major_version": obj.get("webhook_major_version")
                if obj.get("webhook_major_version") is not None
                else 1,
                "webhook_minor_version": obj.get("webhook_minor_version")
                if obj.get("webhook_minor_version") is not None
                else 0,
                "webhook_idempotency_key": obj.get("webhook_idempotency_key"),
                "webhook_timestamp": obj.get("webhook_timestamp"),
                "link_id": obj.get("link_id"),
                "link_tags": obj.get("link_tags"),
                "state": obj.get("state"),
                "error": obj.get("error"),
                "error_message": obj.get("error_message"),
            }
        )
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



# coding: utf-8

"""
    MoneyKit API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2023-02-18
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501



from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Dict, List, Optional, Tuple, Union, Any

try:
    from typing import Annotated
except ImportError:
    from typing_extensions import Annotated

from pydantic import Field
from typing_extensions import Annotated
from pydantic import StrictBool, StrictStr

from typing import Optional

from moneykit.models.get_institutions_response import GetInstitutionsResponse
from moneykit.models.institution import Institution

from moneykit.api_client import ApiClient
from moneykit.api_response import ApiResponse
from moneykit.rest import RESTResponseType


class InstitutionsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def get_institution(
        self,
        institution_id: Annotated[
            StrictStr, Field(description="The institution ID to fetch.")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Institution:
        """/institutions/{institution_id}

        Fetches details about a single institution.

        :param institution_id: The institution ID to fetch. (required)
        :type institution_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_institution_serialize(
            institution_id=institution_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "Institution",
            "401": "Response401GetInstitutionInstitutionsInstitutionIdGet",
            "429": "APIErrorRateLimitExceededResponse",
            "404": "InstitutionErrorNotFoundResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def get_institution_with_http_info(
        self,
        institution_id: Annotated[
            StrictStr, Field(description="The institution ID to fetch.")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Institution]:
        """/institutions/{institution_id}

        Fetches details about a single institution.

        :param institution_id: The institution ID to fetch. (required)
        :type institution_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_institution_serialize(
            institution_id=institution_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "Institution",
            "401": "Response401GetInstitutionInstitutionsInstitutionIdGet",
            "429": "APIErrorRateLimitExceededResponse",
            "404": "InstitutionErrorNotFoundResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def get_institution_without_preload_content(
        self,
        institution_id: Annotated[
            StrictStr, Field(description="The institution ID to fetch.")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """/institutions/{institution_id}

        Fetches details about a single institution.

        :param institution_id: The institution ID to fetch. (required)
        :type institution_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_institution_serialize(
            institution_id=institution_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "Institution",
            "401": "Response401GetInstitutionInstitutionsInstitutionIdGet",
            "429": "APIErrorRateLimitExceededResponse",
            "404": "InstitutionErrorNotFoundResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_institution_serialize(
        self,
        institution_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if institution_id is not None:
            _path_params["institution_id"] = institution_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # authentication setting
        _auth_settings: List[str] = ["OAuth2ClientCredentials"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/institutions/{institution_id}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def get_institutions(
        self,
        name: Annotated[
            Optional[StrictStr],
            Field(
                description="If provided, returns only institutions containing this name (wholly or as a prefix)."
            ),
        ] = None,
        featured: Annotated[
            Optional[StrictBool],
            Field(description="If true, returns only featured institutions."),
        ] = None,
        cursor: Annotated[
            Optional[StrictStr],
            Field(
                description="Cursor to fetch the next set of institutions. (You get this value from the previous call to `/institutions`.)"
            ),
        ] = None,
        limit: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="A limit on the number of institutions to be returned."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetInstitutionsResponse:
        """/institutions

        Fetches a list of institutions, optionally filtered by name.  Results are paginated.

        :param name: If provided, returns only institutions containing this name (wholly or as a prefix).
        :type name: str
        :param featured: If true, returns only featured institutions.
        :type featured: bool
        :param cursor: Cursor to fetch the next set of institutions. (You get this value from the previous call to `/institutions`.)
        :type cursor: str
        :param limit: A limit on the number of institutions to be returned.
        :type limit: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_institutions_serialize(
            name=name,
            featured=featured,
            cursor=cursor,
            limit=limit,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GetInstitutionsResponse",
            "401": "Response401GetInstitutionsInstitutionsGet",
            "429": "APIErrorRateLimitExceededResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def get_institutions_with_http_info(
        self,
        name: Annotated[
            Optional[StrictStr],
            Field(
                description="If provided, returns only institutions containing this name (wholly or as a prefix)."
            ),
        ] = None,
        featured: Annotated[
            Optional[StrictBool],
            Field(description="If true, returns only featured institutions."),
        ] = None,
        cursor: Annotated[
            Optional[StrictStr],
            Field(
                description="Cursor to fetch the next set of institutions. (You get this value from the previous call to `/institutions`.)"
            ),
        ] = None,
        limit: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="A limit on the number of institutions to be returned."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetInstitutionsResponse]:
        """/institutions

        Fetches a list of institutions, optionally filtered by name.  Results are paginated.

        :param name: If provided, returns only institutions containing this name (wholly or as a prefix).
        :type name: str
        :param featured: If true, returns only featured institutions.
        :type featured: bool
        :param cursor: Cursor to fetch the next set of institutions. (You get this value from the previous call to `/institutions`.)
        :type cursor: str
        :param limit: A limit on the number of institutions to be returned.
        :type limit: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_institutions_serialize(
            name=name,
            featured=featured,
            cursor=cursor,
            limit=limit,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GetInstitutionsResponse",
            "401": "Response401GetInstitutionsInstitutionsGet",
            "429": "APIErrorRateLimitExceededResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def get_institutions_without_preload_content(
        self,
        name: Annotated[
            Optional[StrictStr],
            Field(
                description="If provided, returns only institutions containing this name (wholly or as a prefix)."
            ),
        ] = None,
        featured: Annotated[
            Optional[StrictBool],
            Field(description="If true, returns only featured institutions."),
        ] = None,
        cursor: Annotated[
            Optional[StrictStr],
            Field(
                description="Cursor to fetch the next set of institutions. (You get this value from the previous call to `/institutions`.)"
            ),
        ] = None,
        limit: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="A limit on the number of institutions to be returned."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """/institutions

        Fetches a list of institutions, optionally filtered by name.  Results are paginated.

        :param name: If provided, returns only institutions containing this name (wholly or as a prefix).
        :type name: str
        :param featured: If true, returns only featured institutions.
        :type featured: bool
        :param cursor: Cursor to fetch the next set of institutions. (You get this value from the previous call to `/institutions`.)
        :type cursor: str
        :param limit: A limit on the number of institutions to be returned.
        :type limit: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_institutions_serialize(
            name=name,
            featured=featured,
            cursor=cursor,
            limit=limit,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GetInstitutionsResponse",
            "401": "Response401GetInstitutionsInstitutionsGet",
            "429": "APIErrorRateLimitExceededResponse",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_institutions_serialize(
        self,
        name,
        featured,
        cursor,
        limit,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if name is not None:
            _query_params.append(("name", name))

        if featured is not None:
            _query_params.append(("featured", featured))

        if cursor is not None:
            _query_params.append(("cursor", cursor))

        if limit is not None:
            _query_params.append(("limit", limit))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # authentication setting
        _auth_settings: List[str] = ["OAuth2ClientCredentials"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/institutions",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



# coding: utf-8

"""
    MoneyKit API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2023-02-18
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictBool
from pydantic import Field

try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class TransactionsProductSettings(BaseModel):
    """
    TransactionsProductSettings
    """  # noqa: E501

    required: Optional[StrictBool] = Field(
        default=False,
        description="If true, only institutions supporting this product will be available.",
    )
    prefetch: Optional[StrictBool] = Field(
        default=False,
        description="If true, the data will be available as soon as possible after linking, even if `required` is false. If false, the data will be available after the first manual data refresh.",
    )
    extend_history: Optional[StrictBool] = Field(
        default=False,
        description="If true, MoneyKit will attempt to fetch as much transaction history as possible. Not all institutions supply the same extent of transaction history. Generally, however, at least the past 30 days of transactions are available.",
    )
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["required", "prefetch", "extend_history"]

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of TransactionsProductSettings from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
                "additional_properties",
            },
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of TransactionsProductSettings from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "required": obj.get("required")
                if obj.get("required") is not None
                else False,
                "prefetch": obj.get("prefetch")
                if obj.get("prefetch") is not None
                else False,
                "extend_history": obj.get("extend_history")
                if obj.get("extend_history") is not None
                else False,
            }
        )
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



# coding: utf-8

"""
    MoneyKit API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2023-02-18
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date, datetime
from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictStr
from pydantic import Field

try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class InvestmentTransactionResponse(BaseModel):
    """
    InvestmentTransactionResponse
    """  # noqa: E501

    id: StrictStr = Field(description="The unique ID for this investment transaction.")
    account_id: StrictStr = Field(
        description="The ID of the account in which this investment transaction occurred."
    )
    amount: StrictStr = Field(
        description="The total value of this transaction, denominated in account currency.  Positive values indicate         debits (deposits); negative values are credits (withdrawals).  For example, for a `buy` transaction, the amount         will be equal to `-(quantity * price + fees)`."
    )
    var_date: date = Field(
        description="The effective (posted) date of the transaction, in ISO-8601 format.",
        alias="date",
    )
    datetime_: Optional[datetime] = Field(
        alias="datetime",
        default=None,
        description="If the institution has provided the actual time of the transaction, this field             contains the full date and time of the transaction, in ISO-8601 format.  If the time is             not available, this field will be null.             <p>Note that the time is generally reported in the timezone of the institution or the account holder.",
    )
    description: StrictStr = Field(
        description="A normalized, cleaned transaction description suitable for presentation to the end user."
    )
    type: StrictStr = Field(
        description="The type of transaction (buy, sell, cash, fee, transfer).  May include a dotted subtype,         for example, `buy.buy_to_cover` or `fee.transfer_fee`."
    )
    fees: Optional[StrictStr] = Field(
        default=None,
        description="The combined value of any fees applied to the transaction.  Fees are subtracted from the         total transaction amount, and are reported as positive values; refunds are reported as negative values.",
    )
    forex_rate: Optional[StrictStr] = Field(
        default=None,
        description="The currency exchange rate applied.  Only present for transactions involving a security that is         denominated in a currency other than the account currency.",
    )
    price: Optional[StrictStr] = Field(
        default=None,
        description="The price per share, denominated in account currency.  Omitted for transactions not involving         a security.",
    )
    quantity: Optional[StrictStr] = Field(
        default=None,
        description='The units of security (aka "shares") bought, sold, or transferred.  Omitted for transactions         not involving a security.  Positive values indicate shares added to the account.',
    )
    security_id: Optional[StrictStr] = Field(
        default=None,
        description="The MoneyKit ID of the security involved in this transaction (for example, the security bought         or sold, or upon which interest or dividends are paid.",
    )
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = [
        "id",
        "account_id",
        "amount",
        "date",
        "datetime",
        "description",
        "type",
        "fees",
        "forex_rate",
        "price",
        "quantity",
        "security_id",
    ]

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of InvestmentTransactionResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
                "additional_properties",
            },
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of InvestmentTransactionResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "account_id": obj.get("account_id"),
                "amount": obj.get("amount"),
                "date": obj.get("date"),
                "datetime": obj.get("datetime"),
                "description": obj.get("description"),
                "type": obj.get("type"),
                "fees": obj.get("fees"),
                "forex_rate": obj.get("forex_rate"),
                "price": obj.get("price"),
                "quantity": obj.get("quantity"),
                "security_id": obj.get("security_id"),
            }
        )
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



# coding: utf-8

"""
    MoneyKit API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2023-02-18
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictInt
from pydantic import Field
from moneykit.models.transaction import Transaction

try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class GetUserTransactionsResponse(BaseModel):
    """
    GetUserTransactionsResponse
    """  # noqa: E501

    total: StrictInt = Field(description="The total number of results for this query.")
    page: StrictInt = Field(
        description="The page number corresponding to this batch of results."
    )
    size: StrictInt = Field(description="The number of results in this batch.")
    transactions: List[Transaction]
    accounts: Optional[Any]
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = [
        "total",
        "page",
        "size",
        "transactions",
        "accounts",
    ]

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of GetUserTransactionsResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
                "additional_properties",
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in transactions (list)
        _items = []
        if self.transactions:
            for _item in self.transactions:
                if _item:
                    _items.append(_item.to_dict())
            _dict["transactions"] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        # set to None if accounts (nullable) is None
        # and model_fields_set contains the field
        if self.accounts is None and "accounts" in self.model_fields_set:
            _dict["accounts"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of GetUserTransactionsResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "total": obj.get("total"),
                "page": obj.get("page"),
                "size": obj.get("size"),
                "transactions": [
                    Transaction.from_dict(_item) for _item in obj.get("transactions")
                ]
                if obj.get("transactions") is not None
                else None,
            }
        )
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



# coding: utf-8

"""
    MoneyKit API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2023-02-18
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date, datetime
from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictBool, StrictStr
from pydantic import Field
from moneykit.models.transaction_type import TransactionType

try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class Transaction(BaseModel):
    """
    Transaction
    """  # noqa: E501

    transaction_id: StrictStr = Field(description="The unique ID for this transaction.")
    account_id: StrictStr = Field(
        description="The ID of the account in which this transaction occurred."
    )
    amount: StrictStr = Field(
        description="The amount of this transaction, denominated in account currency.  This amount is always         non-negative.  The `type` field indicates whether it is entering or leaving the account."
    )
    type: TransactionType
    currency: StrictStr = Field(
        description="The ISO-4217 currency code of the transaction."
    )
    date_: date = Field(
        description="The effective (posted) date of the transaction, in ISO-8601 format.  For pending transactions,             this date is when the transaction was initiated.",
        alias="date",
    )
    datetime_: Optional[datetime] = Field(
        alias="datetime",
        default=None,
        description="If the institution has provided the actual time of the transaction, this field             contains the full date and time of the transaction, in ISO-8601 format.  If the time is             not available, this field will be null.             <p>Note that the time is generally reported in the timezone of the institution or the account holder.",
    )
    description: Optional[StrictStr] = Field(
        default=None,
        description="A normalized, cleaned transaction description suitable for presentation to the end user.             Commonly this will be the merchant or counterparty name.",
    )
    raw_description: Optional[StrictStr] = Field(
        default=None,
        description="The raw transaction description as provided by the institution, where available.",
    )
    pending: StrictBool = Field(
        description="If true, this transaction is pending or unsettled and has not yet affected the account.         Commonly these are credit card transactions, particularly approvals (holds) such as for hotel or restaurant         reservations placed in advance where the final amount is still to be determined."
    )
    category: Optional[StrictStr] = Field(
        default=None,
        description="The category for this transaction, given as a dotted string indicating a hierarchical         categorization.  See <a href=/pages/categories>Transaction Categories</a> for the list of possible transaction types.",
    )
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = [
        "transaction_id",
        "account_id",
        "amount",
        "type",
        "currency",
        "date",
        "datetime",
        "description",
        "raw_description",
        "pending",
        "category",
    ]

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Transaction from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
                "additional_properties",
            },
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of Transaction from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "transaction_id": obj.get("transaction_id"),
                "account_id": obj.get("account_id"),
                "amount": obj.get("amount"),
                "type": obj.get("type"),
                "currency": obj.get("currency"),
                "date": obj.get("date"),
                "datetime": obj.get("datetime"),
                "description": obj.get("description"),
                "raw_description": obj.get("raw_description"),
                "pending": obj.get("pending"),
                "category": obj.get("category"),
            }
        )
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj
